@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2016 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2012-2017 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -40,6 +40,7 @@
 #include "gs-app.h"
 #include "gs-app-list.h"
 #include "gs-category.h"
+#include "gs-price.h"
 
 G_BEGIN_DECLS
 
@@ -191,11 +192,10 @@ gboolean	 gs_plugin_add_installed		(GsPlugin	*plugin,
  * @cancellable: a #GCancellable, or %NULL
  * @error: a #GError, or %NULL
  *
- * Get the list of pre-downloaded, pre-checked updates, with the write lock
- * held.
+ * Get the list of updates.
  *
- * NOTE: Actually downloading the updates is normally done in
- * gs_plugin_refresh() when called with %GS_PLUGIN_REFRESH_FLAGS_PAYLOAD.
+ * NOTE: Actually downloading the updates can be done in gs_plugin_download_app()
+ * or in gs_plugin_update_app().
  *
  * Plugins are expected to add new apps using gs_app_list_add().
  *
@@ -302,6 +302,25 @@ gboolean	 gs_plugin_add_category_apps		(GsPlugin	*plugin,
 							 GCancellable	*cancellable,
 							 GError		**error);
 
+/**
+ * gs_plugin_add_recent:
+ * @plugin: a #GsPlugin
+ * @age: a number of seconds
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Return all the applications that have had upstream releases recently.
+ *
+ * Plugins are expected to add new apps using gs_app_list_add().
+ *
+ * Returns: %TRUE for success or if not relevant
+ **/
+gboolean	 gs_plugin_add_recent			(GsPlugin	*plugin,
+							 GsAppList	*list,
+							 guint64	 age,
+							 GCancellable	*cancellable,
+							 GError		**error);
+
 /**
  * gs_plugin_add_popular:
  * @plugin: a #GsPlugin
@@ -428,6 +447,30 @@ gboolean	 gs_plugin_refine_app			(GsPlugin	*plugin,
 							 GCancellable	*cancellable,
 							 GError		**error);
 
+/**
+ * gs_plugin_refine_wildcard:
+ * @plugin: a #GsPlugin
+ * @app: a #GsApp
+ * @list: a #GsAppList
+ * @flags: a #GsPluginRefineFlags, e.g. %GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENSE
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Adds applications that match the wildcard specified in @app.
+ *
+ * The general idea is that plugins create and add *new* applications rather
+ * than all trying to fight over the wildcard application.
+ * This allows the plugin loader to filter using the #GsApp priority value.
+ *
+ * Returns: %TRUE for success or if not relevant
+ **/
+gboolean	 gs_plugin_refine_wildcard		(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GsAppList	*list,
+							 GsPluginRefineFlags flags,
+							 GCancellable	*cancellable,
+							 GError		**error);
+
 /**
  * gs_plugin_launch:
  * @plugin: a #GsPlugin
@@ -496,6 +539,27 @@ gboolean	 gs_plugin_update_cancel		(GsPlugin	*plugin,
 							 GCancellable	*cancellable,
 							 GError		**error);
 
+/**
+ * gs_plugin_app_purchase:
+ * @plugin: a #GsPlugin
+ * @app: a #GsApp
+ * @price: a #GsPrice
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Purchase the application.
+ *
+ * NOTE: Once the action is complete, the plugin must set the new state of @app
+ * to %AS_APP_STATE_AVAILABLE.
+ *
+ * Returns: %TRUE for success or if not relevant
+ **/
+gboolean	 gs_plugin_app_purchase			(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GsPrice	*price,
+							 GCancellable	*cancellable,
+							 GError		**error);
+
 /**
  * gs_plugin_app_install:
  * @plugin: a #GsPlugin
@@ -513,8 +577,7 @@ gboolean	 gs_plugin_update_cancel		(GsPlugin	*plugin,
  * to complete.
  *
  * On failure the error message returned will usually only be shown on the
- * console, but it may also be retained on the #GsApp object.
- * The UI code can retrieve the error using gs_app_get_last_error().
+ * console, but they can also be retrieved using gs_plugin_loader_get_events().
  *
  * NOTE: Once the action is complete, the plugin must set the new state of @app
  * to %AS_APP_STATE_INSTALLED.
@@ -543,8 +606,7 @@ gboolean	 gs_plugin_app_install			(GsPlugin	*plugin,
  * to complete.
  *
  * On failure the error message returned will usually only be shown on the
- * console, but it may also be retained on the #GsApp object.
- * The UI code can retrieve the error using gs_app_get_last_error().
+ * console, but they can also be retrieved using gs_plugin_loader_get_events().
  *
  * NOTE: Once the action is complete, the plugin must set the new state of @app
  * to %AS_APP_STATE_AVAILABLE or %AS_APP_STATE_UNKNOWN if not known.
@@ -591,12 +653,15 @@ gboolean	 gs_plugin_app_set_rating		(GsPlugin	*plugin,
  * to complete.
  *
  * On failure the error message returned will usually only be shown on the
- * console, but it may also be retained on the #GsApp object.
- * The UI code can retrieve the error using gs_app_get_last_error().
+ * console, but they can also be retrieved using gs_plugin_loader_get_events().
  *
  * NOTE: Once the action is complete, the plugin must set the new state of @app
  * to %AS_APP_STATE_INSTALLED or %AS_APP_STATE_UNKNOWN if not known.
  *
+ * If %AS_APP_QUIRK_IS_PROXY is set on the application then the actual #GsApp
+ * set in @app will be the related application of the parent. Plugins do not
+ * need to manually iterate on the related list of applications.
+ *
  * Returns: %TRUE for success or if not relevant
  **/
 gboolean	 gs_plugin_update_app			(GsPlugin	*plugin,
@@ -604,6 +669,51 @@ gboolean	 gs_plugin_update_app			(GsPlugin	*plugin,
 							 GCancellable	*cancellable,
 							 GError		**error);
 
+/**
+ * gs_plugin_download_app:
+ * @plugin: a #GsPlugin
+ * @app: a #GsApp
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Downloads the application and any dependencies ready to be installed or
+ * updated.
+ *
+ * Plugins are expected to send progress notifications to the UI using
+ * gs_app_set_progress() using the passed in @app.
+ *
+ * All functions can block, but should sent progress notifications, e.g. using
+ * gs_app_set_progress() if they will take more than tens of milliseconds
+ * to complete.
+ *
+ * If the @app is already downloaded, do not return an error and return %TRUE.
+ *
+ * On failure the error message returned will usually only be shown on the
+ * console, but they can also be retrieved using gs_plugin_loader_get_events().
+ *
+ * Returns: %TRUE for success or if not relevant
+ **/
+gboolean	 gs_plugin_download_app			(GsPlugin	*plugin,
+							 GsApp		*app,
+							 GCancellable	*cancellable,
+							 GError		**error);
+
+/**
+ * gs_plugin_download:
+ * @plugin: a #GsPlugin
+ * @apps: a #GsAppList
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Downloads a list of applications ready to be installed or updated.
+ *
+ * Returns: %TRUE for success or if not relevant
+ **/
+gboolean	 gs_plugin_download			(GsPlugin	*plugin,
+							 GsAppList	*apps,
+							 GCancellable	*cancellable,
+							 GError		**error);
+
 /**
  * gs_plugin_app_upgrade_download:
  * @plugin: a #GsPlugin
@@ -759,21 +869,13 @@ gboolean	 gs_plugin_review_dismiss		(GsPlugin	*plugin,
  * gs_plugin_refresh:
  * @plugin: a #GsPlugin
  * @cache_age: the acceptable cache age in seconds, or MAXUINT for "any"
- * @flags: a bitfield of #GsPluginRefreshFlags, e.g. %GS_PLUGIN_REFRESH_FLAGS_METADATA
  * @cancellable: a #GCancellable, or %NULL
  * @error: a #GError, or %NULL
  *
- * Refreshes the state of all the plugins.
- *
- * The %GS_PLUGIN_REFRESH_FLAGS_METADATA flag can be used to make sure
+ * Refreshes the state of all the plugins. Plugins should make sure
  * there's enough metadata to start the application, for example lists of
  * available applications.
  *
- * The %GS_PLUGIN_REFRESH_FLAGS_PAYLOAD flag should only be used when
- * the session is idle and bandwidth is unmetered as the amount of data
- * and IO may be large.
- * This is used to pre-download package updates and firmware.
- *
  * All functions can block, but should sent progress notifications, e.g. using
  * gs_app_set_progress() if they will take more than tens of milliseconds
  * to complete.
@@ -782,7 +884,6 @@ gboolean	 gs_plugin_review_dismiss		(GsPlugin	*plugin,
  **/
 gboolean	 gs_plugin_refresh			(GsPlugin	*plugin,
 							 guint		 cache_age,
-							 GsPluginRefreshFlags flags,
 							 GCancellable	*cancellable,
 							 GError		**error);
 
@@ -813,6 +914,30 @@ gboolean	 gs_plugin_file_to_app			(GsPlugin	*plugin,
 							 GCancellable	*cancellable,
 							 GError		**error);
 
+/**
+ * gs_plugin_url_to_app:
+ * @plugin: a #GsPlugin
+ * @list: a #GsAppList
+ * @url: a #URL, e.g. "apt://gimp"
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Converts a URL to a #GsApp. It's expected that only one plugin will
+ * match the scheme of @url and that a single #GsApp will be in the returned
+ * list. If no plugins can handle the file, the list will be empty.
+ *
+ * For example, the apt plugin can turn apt://gimp into a application.
+ *
+ * Plugins are expected to add new apps using gs_app_list_add().
+ *
+ * Returns: %TRUE for success or if not relevant
+ **/
+gboolean	 gs_plugin_url_to_app			(GsPlugin	*plugin,
+							 GsAppList	*list,
+							 const gchar	*url,
+							 GCancellable	*cancellable,
+							 GError		**error);
+
 /**
  * gs_plugin_update:
  * @plugin: a #GsPlugin
